/* packet-dcerpc-dcom.c
 * Routines for DCOM generics
 * Copyright 2002, Ulf Lamping <ulf.lamping@siemens.com>
 *
 * $Id: packet-dcerpc-dcom.c,v 1.6 2002/08/02 23:35:48 jmayer Exp $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/* A few words about DCOM: DCOM sits on top of DCERPC
 *
 * DCOM is called by DCERPC request and response calls.
 * It uses a small header after the DCERPC calls.
 * (for a DCERPC request call called "this", for DCERPC response called "that")
 *
 * DCOM defines itself some interfaces: oxid, remact, remunk and others
 *
 * Implemented is currently "only" a static dissection of packet fields
 * (no "object recognition" included)
 *
 *
 * Hint: The DCOM interface names can be found in the windows registry at:
 * "HKEY_CLASSES_ROOT\Interface"
 *
 * Ressources on the web: 
 *
 * "Understanding the DCOM Wire Protocol by Analyzing Network Data Packets"
 * http:// www.microsoft.com/msj/0398/dcom.htm
 *
 * "Distributed Component Object Model Protocol -- DCOM/1.0"
 * http://www.microsoft.com/com/resources/specs.asp (link is down)
 * 
 */

/* Files involved dissecting DCOM:
 *
 * packet-dcom.c: generic DCOM things (this, that, ...)
 * packet-dcom-types.c: generic DCOM datatypes (DWORD, VARIANT, ...)
 *
 * DCOM common Interfaces:
 * packet-dcom-oxid.c:     IOXIDResolver
 * packet-dcom-remact.c:   IRemoteActivation
 * packet-dcom-remunk.c:   IRemUnknown, IRemUnknown2
 * packet-dcom-dispatch.c: IDispatch
 * packet-dcom-sysact.c:   ISystemActivator
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif

#include <glib.h>
#include <epan/packet.h>
#include "packet-dcerpc.h"
#include "packet-dcom.h"
#include "prefs.h"


static int proto_dcom = -1;
void proto_reg_handoff_dcom(void);

int dcom_prefs_display_unmarshalling_details = FALSE;


gint ett_dcom_this = -1;
static int hf_dcom_this = -1;
static int hf_dcom_this_version_major = -1;
static int hf_dcom_this_version_minor = -1;
static int hf_dcom_this_flags = -1;
static int hf_dcom_this_res = -1;
static int hf_dcom_this_cid = -1;
static int hf_dcom_this_extensions = -1;

gint ett_dcom_that = -1;
static int hf_dcom_that = -1;
static int hf_dcom_that_flags = -1;
static int hf_dcom_that_extensions = -1;



/* build dcom "this" subtree */
int
dissect_dcom_this(tvbuff_t *tvb, int offset,
	packet_info *pinfo, proto_tree *tree, char *drep)
{
	guint16 u16VersionMajor;
	guint16 u16VersionMinor;
	guint32 u32Flags;
	guint32 u32Res;
	guint32 u32UuidExtends;
	e_uuid_t uuidCausality;
	proto_item *sub_item;
	proto_tree *sub_tree;
	guint32 u32SubStart;
	

	/* not enough buffer for this? */
	if (tvb_length_remaining (tvb, offset) < 32) {
		return offset;
	}	

	sub_item = proto_tree_add_item(tree, hf_dcom_this, tvb, offset, 0, FALSE);
	sub_tree = proto_item_add_subtree(sub_item, ett_dcom_this);

	offset = dissect_dcom_COMVERSION(tvb, offset, pinfo, sub_tree, drep,
				&u16VersionMajor, &u16VersionMinor);
	u32SubStart = offset - 4;

	offset = dissect_dcom_DWORD(tvb, offset, pinfo, sub_tree, drep, 
                        hf_dcom_this_flags, &u32Flags);
	offset = dissect_dcom_DWORD(tvb, offset, pinfo, sub_tree, drep, 
                        hf_dcom_this_res, &u32Res);

	offset = dissect_dcom_UUID(tvb, offset, pinfo, sub_tree, drep, 
                        hf_dcom_this_cid, &uuidCausality);

	/* ToBeDone: extend this ;-) */
	offset = dissect_dcom_DWORD(tvb, offset, pinfo, sub_tree, drep, 
                        hf_dcom_this_extensions, &u32UuidExtends);

	/* update subtree header */
	proto_item_append_text(sub_item, ", V%u.%u, Causality ID: %s", 
		u16VersionMajor, u16VersionMinor, dcerpc_uuid_to_str(&uuidCausality));
	proto_item_set_len(sub_item, offset - u32SubStart);

	return offset;
}


/* build dcom "that" subtree */
int
dissect_dcom_that(tvbuff_t *tvb, int offset,
	packet_info *pinfo, proto_tree *tree, char *drep) {
	guint32 u32Flags;
	guint32 u32Extensions;
	proto_item *sub_item;
	proto_tree *sub_tree;
	guint32 u32SubStart;


	sub_item = proto_tree_add_item(tree, hf_dcom_that, tvb, offset, 0, FALSE);
	sub_tree = proto_item_add_subtree(sub_item, ett_dcom_that);

	offset = dissect_dcom_DWORD(tvb, offset, pinfo, sub_tree, drep, 
                        hf_dcom_that_flags, &u32Flags);
	u32SubStart = offset - 4;

	/* ToBeDone: extend that ;-) */
	offset = dissect_dcom_DWORD(tvb, offset, pinfo, sub_tree, drep, 
                        hf_dcom_that_extensions, &u32Extensions);
	
	/* update subtree header */
	proto_item_set_len(sub_item, offset - u32SubStart);

	return offset;
}



/* "pseudo" sub dissector table for DCOM "pre" interface */
static dcerpc_sub_dissector dcom_pre_dissectors[] = {
    { 0, "DCOM", dissect_dcom_this, dissect_dcom_that },
    { 0, NULL, NULL, NULL },
};



/* handle dcom request, if no special dissector is available (yet) */
int
dissect_dcom_tobedone_rqst(tvbuff_t *tvb, int offset,
	packet_info *pinfo, proto_tree *tree, char *drep)
{
	guint32	u32Length;


	/* if everything left, display it as stub data hexdump */
    u32Length = tvb_length_remaining(tvb, offset);
	if (u32Length) {
		offset = dissect_dcom_tobedone_data(tvb, offset, pinfo, tree, drep, u32Length);
	}
	
	if (check_col(pinfo->cinfo, COL_INFO)) {
	  col_append_fstr(pinfo->cinfo, COL_INFO, " dissector ToBeDone (%u bytes Stub-Data)", u32Length);
	}

	return offset;
}


/* handle dcom response, if no special dissector is available (yet) */
int 
dissect_dcom_tobedone_resp(tvbuff_t *tvb, int offset,
	packet_info *pinfo, proto_tree *tree, char *drep)
{
	guint32	u32Length;


    u32Length = tvb_length_remaining (tvb, offset);
	if (u32Length) {
		offset = dissect_dcom_tobedone_data(tvb, offset, pinfo, tree, drep, u32Length);
	}
	
	if (check_col(pinfo->cinfo, COL_INFO)) {
	  col_append_fstr(pinfo->cinfo, COL_INFO, " dissector ToBeDone (%u bytes Stub-Data)", u32Length);
	}

	return offset;
}


/* handle simple dcom request, "DCOMthis" only */
int 
dissect_dcom_simple_rqst(tvbuff_t *tvb, int offset,
	packet_info *pinfo, proto_tree *tree, char *drep)
{

	return offset;
}


/* handle simple dcom response, "DCOMthat" and returned HRESULT only */
int 
dissect_dcom_simple_resp(tvbuff_t *tvb, int offset,
	packet_info *pinfo, proto_tree *tree, char *drep)
{
	guint32 u32HResult;


	offset = dissect_dcom_HRESULT(tvb, offset, pinfo, tree, drep, 
                    &u32HResult);

	if (check_col(pinfo->cinfo, COL_INFO)) {
	  col_append_fstr(pinfo->cinfo, COL_INFO, " -> %s", 
		val_to_str(u32HResult, dcom_hresult_vals, "Unknown (0x%08x)") );
	}

	return offset;
}



void
proto_register_dcom (void)
{

	static hf_register_info hf_dcom_this_array[] = {
		{ &hf_dcom_this,
		{ "DCOM, ORPCThis", "dcom.this", FT_NONE, BASE_HEX, NULL, 0x0, "", HFILL }},
		{ &hf_dcom_this_version_major,
		{ "VersionMajor", "dcom.this_version_major", FT_UINT16, BASE_DEC, NULL, 0x0, "", HFILL }},
		{ &hf_dcom_this_version_minor,
		{ "VersionMinor", "dcom.this_version_minor", FT_UINT16, BASE_DEC, NULL, 0x0, "", HFILL }},
		{ &hf_dcom_this_flags,
		{ "Flags", "dcom.this_flags", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
		{ &hf_dcom_this_res,
		{ "Reserved", "dcom.this_res", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
        { &hf_dcom_this_cid,
        { "Causality ID", "dcom.this_uuid", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
		{ &hf_dcom_this_extensions,
		{ "Extensions", "dcom.this_uuid_extensions", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }}
	};
	
	static hf_register_info hf_dcom_that_array[] = {
		{ &hf_dcom_that,
		{ "DCOM, ORPCThat", "dcom.that", FT_NONE, BASE_HEX, NULL, 0x0, "", HFILL }},
		{ &hf_dcom_that_flags,
		{ "Flags", "dcom.that_flags", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
		{ &hf_dcom_that_extensions,
		{ "Extensions", "dcom.that.extensions", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }}
	};
	

	static gint *ett_dcom[] = {
		&ett_dcom_this,
		&ett_dcom_that,
	};

	module_t *dcom_module; 


	/* currently, the DCOM protocol "itself" is not implemented */
	/* we only need this, to register some elements */
	proto_dcom = proto_register_protocol ("DCOM", "DCOM", "dcom");
    proto_register_field_array(proto_dcom, hf_dcom_this_array, array_length(hf_dcom_this_array));
    proto_register_field_array(proto_dcom, hf_dcom_that_array, array_length(hf_dcom_that_array));
	proto_register_subtree_array (ett_dcom, array_length (ett_dcom));

	dcom_init_types(proto_dcom);


	/* preferences */
	dcom_module = prefs_register_protocol(proto_dcom, proto_reg_handoff_dcom);

	prefs_register_bool_preference(dcom_module, "display_unmarshalling_details", 
		"Display DCOM unmarshalling details", 
		"Display some DCOM unmarshalled fields "
		"usually hidden",
		&dcom_prefs_display_unmarshalling_details);
}


void dcom_register_server_interface(int proto, int ett, e_uuid_t *uuid, guint16 ver, 
					dcerpc_sub_dissector *sub_dissectors, int opnum_hf) {

	/*
	 * Currently, DCOM this and that dissectors are called 
	 *    BEFORE the specific user-interface dissector.
	 *
	 * ToBeDone: There should be the following process to handle a DCOM interface:
	 *    (must be implemented here and in packet-dcerpc.c)
	 *
	 * Registration:
	 * -a DCOM user-interface should register itself calling this function here
	 * -DCOM should register its own (singleton) handler 
	 *    with the given interface info as a DCERPC protocol
	 *
	 * Callback:
	 * -DCERPC should always call the DCOM "generic" dissector, which
	 *    has to dissect the THIS- or THAT-pointer, and then call the specific
	 *    sub_dissector function of the user-interface registered here.
	 *
	 */
	dcerpc_init_uuid_with_preprocess(proto, ett, uuid, ver, sub_dissectors, opnum_hf, dcom_pre_dissectors);
}


void dcom_register_server_coclass(int proto, int ett, e_uuid_t *uuid, guint16 ver, 
					dcerpc_sub_dissector *sub_dissectors, int opnum_hf) {

	/* ToBeDone: this must be simply the name registration of the UUID,
	 * not a whole sub_dissector registration */
	dcerpc_init_uuid (proto, ett, uuid, ver, sub_dissectors, opnum_hf);
}


void
proto_reg_handoff_dcom (void)
{

	/* Currently, we have nothing to register for DCOM */
}

